%    <CustomTools>
%      <Menu>
%        <Item name="Custom Filter" icon="Matlab" tooltip="XT Custom Filter">
%          <Command>MatlabXT::XT_CustomFilter(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
% 
%
%  Description:
%   CC 20171021
%   Modified from Channel Arithmetics XT.
%
%   This XT allows user to apply Matlab built-in or custom made 2D image filter on specific channel and generate a new filtered channel.    
%   
%   Built-in filter example: In the dialog window, type
%       imboxfilt(ch1)
%   will implement 2-D box filtering to all slices in ch1 and create a new
%   channel
%
%   Custom filter example: user makes a filter function 
%   J = NewFilter(I,arg1,arg2), where I is input image and J is filtered image 
%   In the dialog window, type
%       NewFilter(ch2,arg1,arg2)
%   will create filtered ch2 as a new channel
%
%   ***the script can be modified for 3D image filter 
%   ***NewFilter.m file has to be in XT folder 

function XT_CustomFilter(aImarisApplicationID)
  % connect to Imaris interface
  if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
      aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
  else
    vImarisApplication = aImarisApplicationID;
  end

  % need a dataset
  if isempty(vImarisApplication.GetDataSet())
    msgbox('An image is needed for this XTension');
    return;
  end
  
  %dialog to select channel for filter
  vAnswer = inputdlg({sprintf('Type in the filter function, with input image = ch1, ch2,...\n\nExample: imboxfilt(ch1)')}, ...
    'Custom Filter', 1, {'imboxfilt(ch1)'});  
  if isempty(vAnswer), return, end

  % get DataSet
  vDataSet = vImarisApplication.GetDataSet.Clone;

  vLastC = vDataSet.GetSizeC;
  aSizeX = vImarisApplication.GetDataSet.GetSizeX;
  aSizeY = vImarisApplication.GetDataSet.GetSizeY;
  vDataSet.SetSizeC(vLastC + 1);
  vMin = vDataSet.GetChannelRangeMin(0);
  vMax = vDataSet.GetChannelRangeMax(0);
  vDataSet.SetChannelRange(vLastC, vMin, vMax);

  vProgressTotalCount = vDataSet.GetSizeT*vDataSet.GetSizeZ;
  vProgressDisplay = waitbar(0, 'Custom Filter');
  vProgressCount = 0;

  for vTime = 1:vDataSet.GetSizeT
    for vSlice = 1:vDataSet.GetSizeZ
    
        for vChannel = 1:vLastC
            % works on double to allow division and prevent overflows
            eval(sprintf('ch%i = double(vDataSet.GetDataSliceFloats(vSlice-1, vChannel-1, vTime-1));', vChannel));
        end
    
        try
            vData = eval(vAnswer{1});
                        
            catch er
                close(vProgressDisplay);
                msgbox(sprintf(['Error while evaluating the expression.\n\n', ...
                'Possible causes: invalid variable names (ch1, ch2, ...),\n\n', er.message]));
        return;
        end
        
        try
            if strcmp(vDataSet.GetType,'eTypeUInt8')
                vDataSet.SetDataSliceBytes(uint8(vData),vSlice-1,vLastC,vTime-1);
            elseif strcmp(vDataSet.GetType,'eTypeUInt16')
                vDataSet.SetDataSliceShorts(uint16(vData),vSlice-1,vLastC,vTime-1);
            elseif strcmp(vDataSet.GetType,'eTypeFloat')
                vDataSet.SetDataSliceFloats(single(vData),vSlice-1,vLastC,vTime-1);
            end
            catch er
                close(vProgressDisplay);
                msgbox(sprintf(['The result of the expression is not a valid dataset.\n\n', ...
                'Possible causes: invalid result size.\n\n', er.message]));
            return
        end

    vProgressCount = vProgressCount + 1;
    waitbar(vProgressCount/vProgressTotalCount, vProgressDisplay);
    end
   end

vImarisApplication.SetDataSet(vDataSet);
close(vProgressDisplay);